/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
 */

package org.gfxtest.framework;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;



/**
 * This class contains static method used by the various tests which checks if
 * clipping is working correctly.
 * 
 * @author Pavel Tisnovsky
 */
public class CommonClippingOperations
{
    /**
     * Color used for rendering shape which is drawn along clip region.
     */
    public static final Color CLIP_SHAPE_COLOR = Color.MAGENTA.darker();

    /**
     * Radius of corners of the round rectangle.
     */
    public static final int ROUND_RECTANGLE_ARC_RADIUS = 300;

    /**
     * Compute width of the clip region.
     * 
     * @param image
     *            work image
     * @return width of the clip region
     */
    private static int computeClipRegionWidth(TestImage image)
    {
        // compute width first
        int width = 2 * image.getWidth() / 3;
        // it's best to align clip area with the grid
        width = width - (width % image.getGrid());
        return width - 1;
    }

    /**
     * Compute height of the clip region.
     * 
     * @param image
     *            work image
     * @return height of the clip region
     */
    private static int computeClipRegionHeight(TestImage image)
    {
        // compute height first
        int height = 2 * image.getHeight() / 3;
        // it's best to align clip area with the grid
        height = height - (height % image.getGrid());
        return height - 1;
    }

    /**
     * Render the rectangle around the clip area. Clip area could have different
     * shape but it's area should lie inside this rectangle.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    public static void renderClipRectangle(TestImage image, Graphics2D graphics2d)
    {
        // compute size of clip area
        final int width = computeClipRegionWidth(image) + 1;
        final int height = computeClipRegionHeight(image) + 1;
        // color used for clipping areas
        graphics2d.setColor(CLIP_SHAPE_COLOR);
        // draw rectangle around the clip area
        graphics2d.drawRect(0, 0, width, height);
    }

    /**
     * Render the round rectangle around the clip area.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    public static void renderClipRoundRectangle(TestImage image, Graphics2D graphics2d)
    {
        // compute size of clip area
        final int width = computeClipRegionWidth(image) + 1;
        final int height = computeClipRegionHeight(image) + 1;
        // color used for clipping areas
        graphics2d.setColor(CLIP_SHAPE_COLOR);
        // draw rounded rectangle around the clip area
        graphics2d.drawRoundRect(0, 0, width, height, ROUND_RECTANGLE_ARC_RADIUS, ROUND_RECTANGLE_ARC_RADIUS);
    }

    /**
     * Render the ellipse around the clip area. Clip area could have different
     * shape but should be inside this ellipse.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    public static void renderClipEllipse(TestImage image, Graphics2D graphics2d)
    {
        // compute size of clip area
        final int width = computeClipRegionWidth(image) + 1;
        final int height = computeClipRegionHeight(image) + 1;
        // color used for clipping areas
        graphics2d.setColor(CLIP_SHAPE_COLOR);
        // draw ellipse around the clip area
        graphics2d.drawOval(0, 0, width, height);
    }

    /**
     * Render convex polygon around the clip area. Clip area could have different
     * shape but should be inside this shape.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    public static void renderConvexClipPolygon(TestImage image, Graphics2D graphics2d)
    {
        // compute size of clip area
        final int width = computeClipRegionWidth(image) + 1;
        final int height = computeClipRegionHeight(image) + 1;
        // color used for clipping areas
        graphics2d.setColor(CLIP_SHAPE_COLOR);
        // draw polygon around the clip area
        graphics2d.drawPolygon(createConvexClippingPolygon(width, height));
    }

    /**
     * Render concave polygon around the clip area. Clip area could have different
     * shape but should be inside this shape.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    public static void renderConcaveClipPolygon(TestImage image, Graphics2D graphics2d)
    {
        // compute size of clip area
        final int width = computeClipRegionWidth(image) + 1;
        final int height = computeClipRegionHeight(image) + 1;
        // color used for clipping areas
        graphics2d.setColor(CLIP_SHAPE_COLOR);
        // draw polygon around the clip area
        graphics2d.drawPolygon(createConcaveClippingPolygon(width, height));
    }

    /**
     * Render the polygon around the clip area. Clip area could have different
     * shape but should be inside this shape.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    public static void renderSelfIntersectingClipPolygon(TestImage image, Graphics2D graphics2d)
    {
        // compute size of clip area
        final int width = computeClipRegionWidth(image) + 1;
        final int height = computeClipRegionHeight(image) + 1;
        // color used for clipping areas
        graphics2d.setColor(CLIP_SHAPE_COLOR);
        // draw polygon around the clip area
        graphics2d.drawPolygon(createSelfIntersectingClippingPolygon(width, height));
    }

    /**
     * Create clip region using rectangle area (not rectangular shape)
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    public static void createClipUsingRectangleArea(TestImage image, Graphics2D graphics2d)
    {
        // compute size of clip area
        final int width = computeClipRegionWidth(image);
        final int height = computeClipRegionHeight(image);
        // apply clip rectangle area
        graphics2d.setClip(1, 1, width, height);
    }

    /**
     * Create clip region using rectangular shape (not rectangle area).
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    public static void createClipUsingRectangleShape(TestImage image, Graphics2D graphics2d)
    {
        // compute size of clip area
        final int width = computeClipRegionWidth(image);
        final int height = computeClipRegionHeight(image);
        // create clip shape
        Shape clipShape = new Rectangle2D.Float(1, 1, width, height);
        // and apply the created shape as clipping region
        graphics2d.setClip(clipShape);
    }

    /**
     * Create clip region using rounded rectangular shape (not rectangle area).
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    public static void createClipUsingRoundRectangleShape(TestImage image, Graphics2D graphics2d)
    {
        // compute size of clip area
        final int width = computeClipRegionWidth(image);
        final int height = computeClipRegionHeight(image);
        // create clip shape
        Shape clipShape = new RoundRectangle2D.Float(1, 1, width, height, ROUND_RECTANGLE_ARC_RADIUS, ROUND_RECTANGLE_ARC_RADIUS);
        // and apply the created shape as clipping region
        graphics2d.setClip(clipShape);
    }

    /**
     * Create clip region using elliptic shape.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    public static void createClipUsingEllipseShape(TestImage image, Graphics2D graphics2d)
    {
        // compute size of clip area
        final int width = computeClipRegionWidth(image);
        final int height = computeClipRegionHeight(image);
        // create clip shape
        Shape clipShape = new Ellipse2D.Float(1, 1, width, height);
        // and apply the created shape as clipping region
        graphics2d.setClip(clipShape);
    }

    /**
     * Create clip region using convex polygonal shape.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    public static void createClipUsingConvexPolygonalShape(TestImage image, Graphics2D graphics2d)
    {
        // compute size of clip area
        final int width = computeClipRegionWidth(image);
        final int height = computeClipRegionHeight(image);
        // create clip shape
        Polygon polygon = createConvexClippingPolygon(width, height);
        // and apply the created shape as clipping region
        graphics2d.setClip(polygon);
    }

    /**
     * Create clip region using concave polygonal shape.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    public static void createClipUsingConcavePolygonalShape(TestImage image, Graphics2D graphics2d)
    {
        // compute size of clip area
        final int width = computeClipRegionWidth(image);
        final int height = computeClipRegionHeight(image);
        // create clip shape
        Polygon polygon = createConcaveClippingPolygon(width, height);
        // and apply the created shape as clipping region
        graphics2d.setClip(polygon);
    }

    /**
     * Create clip region using self-intersecting polygonal shape.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    public static void createClipUsingSelfIntersectingPolygonalShape(TestImage image, Graphics2D graphics2d)
    {
        // compute size of clip area
        final int width = computeClipRegionWidth(image);
        final int height = computeClipRegionHeight(image);
        // create clip shape
        Polygon polygon = createSelfIntersectingClippingPolygon(width, height);
        // and apply the created shape as clipping region
        graphics2d.setClip(polygon);
    }

    /**
     * Create clipping polygon with given width and height. Please note that the
     * polygon has the following shape:
     * 
     * <pre>
     *    ____
     *   /    \
     *  /      \
     *  \      /
     *   \____/
     * </pre>
     * 
     * @param width
     *            image width
     * @param height
     *            image height
     * @return new clipping polygon
     */
    private static Polygon createConvexClippingPolygon(final int width, final int height)
    {
        Polygon polygon = new Polygon();
        // add five vertexes
        polygon.addPoint(1, height >> 1);
        polygon.addPoint(width / 3, 1);
        polygon.addPoint(2 * width / 3, 1);
        polygon.addPoint(width, height >> 1);
        polygon.addPoint(2 * width / 3, height);
        polygon.addPoint(width / 3, height);
        return polygon;
    }

    /**
     * Create clipping polygon with given width and height. Please note that the
     * polygon has the following shape:
     * 
     * <pre>
     *  ______
     *  \    /
     *   \  /
     *   /  \
     *  /____\ 
     * </pre>
     * 
     * @param width
     *            image width
     * @param height
     *            image height
     * @return new clipping polygon
     */
    private static Polygon createConcaveClippingPolygon(final int width, final int height)
    {
        Polygon polygon = new Polygon();
        // add five vertexes
        polygon.addPoint(1, 1);
        polygon.addPoint(width, 1);
        polygon.addPoint(10 + (width >> 1), height >> 1);
        polygon.addPoint(width, height);
        polygon.addPoint(1, height);
        polygon.addPoint((width >> 1) - 10, height >> 1);
        return polygon;
    }

    /**
     * Create clipping polygon with given width and height. Please note that the
     * polygon has the following shape:
     * 
     * <pre>
     * |\    /|
     * | \  / |
     * |  ><  |
     * | /  \ |
     * |/    \|
     * </pre>
     * 
     * @param width
     *            image width
     * @param height
     *            image height
     * @return new clipping polygon
     */
    private static Polygon createSelfIntersectingClippingPolygon(final int width, final int height)
    {
        Polygon polygon = new Polygon();
        // add four vertexes
        polygon.addPoint(1, 1);
        polygon.addPoint(width, height);
        polygon.addPoint(width, 1);
        polygon.addPoint(1, height);
        return polygon;
    }

}
