/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.framework;

import java.util.Date;
import java.util.HashMap;
import java.util.Locale;

/**
 * Simple logger that writes everything out to standard output and can use color
 * codes for ANSI-compatible terminals.
 *
 * @author Pavel Tisnovsky
 */
@SuppressWarnings("nls")
public class Log
{
    static HashMap<String, String> colors = new HashMap<String, String>(10);

    static
    {
        colors.put("RED", "31");
        colors.put("GREEN", "32");
        colors.put("ORANGE", "33");
        colors.put("BLUE", "34");
        colors.put("VIOLET", "35");
        colors.put("MAGENTA", "35");
        colors.put("LIGHT_BLUE", "36");
        colors.put("GRAY", "38");
        colors.put("GREY", "38");
    }

    /**
     * Current indentation level. Increased in logBegin() and decreased in
     * logEnd() methods.
     */
    private int indent = 0;

    /**
     * Indentation level rate.
     */
    private int indentDelta = 2;

    /**
     * Name of class containing Log instance.
     */
    private String className = null;

    /**
     * Whether to use colorified output. Useful when logging to terminal, less
     * useful when log is redirected to a file.
     */
    private boolean useColors;

    /**
     * Constructor.
     *
     * @param className name of class containing Log instance.
     * @param useColors whether to use colorified output.
     */
    public Log(String className, boolean useColors)
    {
        this.className = className;
        this.useColors = useColors;
    }

    public void log(String str)
    {
        this.log(this.className, str);
    }

    public void log(String clsName, String message)
    {
        String str = message;
        StringBuffer spaces = new StringBuffer();
        for (int i = 0; i < this.indent; i++)
        {
            spaces.append(' ');
        }

        StringBuffer output = new StringBuffer();
        if (this.useColors)
        {
            str = "$GREEN$GfxTest  $LIGHT_BLUE$" + this.getDate() + "  $ORANGE$" + String.format("%-40s", clsName)
                    + spaces + "  $GRAY$" + str;
        }
        else
        {
            str = "GfxTest  " + this.getDate() + "  " +  String.format("%-40s", clsName) + spaces + "  " + str;
        }

        String[] pieces = str.split("\\$");

        for (String piece : pieces)
        {
            if (colors.containsKey(piece.toUpperCase(Locale.ENGLISH)))
            {
                if (this.useColors)
                {
                    output.append("\033[0;" + colors.get(piece.toUpperCase(Locale.ENGLISH)) + "m");
                }
            }
            else
            {
                output.append(piece);
            }
        }
        // now it is possible to output new string to a standard output
        if (this.useColors)
        {
            System.out.println(output.append("\033[0m"));
        }
        else
        {
            System.out.println(output);
        }
    }

    public void logSet(String str, Object o)
    {
        String value = o == null ? "$RED$null" : "$GRAY$" + o.toString();
        this.log("$LIGHT_BLUE$" + str + "$ORANGE$ = " + value);
    }

    public void logBegin(String clsName, String str)
    {
        this.log(clsName, str + " $GREEN$begin");
        this.indent += this.indentDelta;
    }

    public void logBegin(String str)
    {
        this.logBegin(this.className, str);
    }

    public void logEnd(String clsName, String str)
    {
        if (this.indent >= this.indentDelta)
        {
            this.indent -= this.indentDelta;
        }
        this.log(clsName, str + " $VIOLET$end");
    }

    public void logEnd(String str)
    {
        this.logEnd(this.className, str);
    }

    public void logError(String str)
    {
        this.logError(this.className, str);
    }

    public void logError(String clsName, String str)
    {
        this.log(clsName, "$RED$error $GRAY$" + str);
    }

    public void logWarning(String formatStr, Object... o)
    {
        this.log(this.className, "$MAGENTA$warning $GRAY$" + String.format(formatStr, o));
    }

    public void logProcess(String formatStr, Object... o)
    {
        this.log(this.className, "$ORANGE$processing: $GRAY$" + String.format(formatStr, o));
    }

    public void logConfig(String formatStr,  Object... o)
    {
        this.log(this.className, "$ORANGE$config: $GRAY$" + String.format(formatStr, o));
    }

    public static void main(String[] args)
    {
        System.out.println("test start");
        System.out.println("test end");
    }

    @SuppressWarnings({ "deprecation", "boxing" })
    private String getDate()
    {
        // vytvorime pomocny buffer, do ktereho seskladame retezec
        Date date = new Date();
        return String.format("%04d-%02d-%02d  %02d:%02d:%02d", 1900 + date.getYear(), date.getMonth(), date.getDay(),
                date.getHours(), date.getMinutes(), date.getSeconds());
    }

}
