/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.framework;

import java.awt.TexturePaint;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;



/**
 * This class contains static method used for creating various procedural
 * textures.
 * 
 * @author Pavel Tisnovsky
 */
public class ProceduralTextureFactory
{
    /**
     * Image used for creating checker texture.
     */
    private static BufferedImage checkerTexture = null;

    /**
     * Image used for creating diagonal checker texture.
     */
    private static BufferedImage diagonalCheckerTexture = null;

    /**
     * Image used for creating grid texture.
     */
    private static BufferedImage gridTexture = null;

    /**
     * Image used for creating diagonal grid texture.
     */
    private static BufferedImage diagonalGridTexture = null;

    /**
     * Image used for creating texture containing horizontal stripes.
     */
    private static BufferedImage horizontalStripesTexture = null;

    /**
     * Image used for creating texture containing vertical stripes.
     */
    private static BufferedImage verticalStripesTexture = null;

    /**
     * Image used for creating texture containing horizontal color stripes.
     */
    private static BufferedImage horizontalColorStripesTexture = null;

    /**
     * Image used for creating texture containing vertical color stripes.
     */
    private static BufferedImage verticalColorStripesTexture = null;

    /**
     * Image used for creating texture containing diagonal stripes.
     */
    private static BufferedImage diagonalStripesTexture = null;

    /**
     * Image used for creating texture containing color dots.
     */
    private static BufferedImage colorDotsTexture = null;

    /**
     * Image used for creating texture containing various RGB colors.
     */
    private static BufferedImage RGBTexture1 = null;

    /**
     * Image used for creating texture containing various RGB colors.
     */
    private static BufferedImage RGBTexture2 = null;

    /**
     * Image used for creating texture containing various RGB colors.
     */
    private static BufferedImage RGBTexture3 = null;

    /**
     * Image used for creating texture containing various RGB colors.
     */
    private static BufferedImage RGBTexture4 = null;

    /**
     * Image used for creating texture containing various RGB colors.
     */
    private static BufferedImage RGBTexture5 = null;

    /**
     * Image used for creating texture containing various RGB colors.
     */
    private static BufferedImage RGBTexture6 = null;

    /**
     * Eight basic colors.
     */
    private static final int[] EIGHT_BASIC_COLOR_PALETTE = {
        0xff000000,
        0xff0000ff,
        0xff00ffff,
        0xff00ff00,
        0xffffff00,
        0xffff0000,
        0xffff00ff,
        0xffffffff,
    };

    /**
     * Create checker texture where tile sizes are based on grid size.
     * 
     * @param gridSize
     *            size of a grid
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @return buffered image containing checker texture
     */
    private static BufferedImage checkerTextureFactory(int gridSize, int width, int height)
    {
        // used by test for a tile colors
        int doubledGridSize = gridSize << 1;

        // if the checker texture already exists, return it
        if (checkerTexture != null)
        {
            return checkerTexture;
        }

        // create new texture instead
        checkerTexture = new BufferedImage(width, height, BufferedImage.TYPE_BYTE_BINARY);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each tile
                boolean evenColumn = (x % doubledGridSize) >= gridSize;
                boolean evenRow = (y % doubledGridSize) >= gridSize;
                // compute color for each pixel
                int color = evenColumn ^ evenRow ? 0 : -1;
                checkerTexture.setRGB(x, y, color);
            }
        }
        return checkerTexture;
    }

    /**
     * Create diagonal checker texture where tile sizes are based on grid size.
     * 
     * @param gridSize
     *            size of a grid
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @return buffered image containing checker texture
     */
    private static BufferedImage diagonalCheckerTextureFactory(int gridSize, int width, int height)
    {
        // used by test for a tile colors
        int doubledGridSize = gridSize << 1;

        // used for computing checker texture
        int width2 = width << 1;

        // if the checker texture already exists, return it
        if (diagonalCheckerTexture != null)
        {
            return diagonalCheckerTexture;
        }

        // create new texture instead
        diagonalCheckerTexture = new BufferedImage(width, height, BufferedImage.TYPE_BYTE_BINARY);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each tile
                boolean evenColumn = ((x + y) % doubledGridSize) >= gridSize;
                boolean evenRow = ((width2 + x - y) % doubledGridSize) >= gridSize;
                // compute color for each pixel
                int color = evenColumn ^ evenRow ? 0 : -1;
                diagonalCheckerTexture.setRGB(x, y, color);
            }
        }
        return diagonalCheckerTexture;
    }

    /**
     * Create grid texture where tile sizes are based on grid size.
     * 
     * @param gridSize
     *            size of a grid
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @return buffered image containing grid texture
     */
    private static BufferedImage gridTextureFactory(int gridSize, int width, int height)
    {
        // used by test for a tile colors
        int doubledGridSize = gridSize << 1;

        // if the grid texture already exists, return it
        if (gridTexture != null)
        {
            return gridTexture;
        }

        // create new texture instead
        gridTexture = new BufferedImage(width, height, BufferedImage.TYPE_BYTE_BINARY);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each tile
                boolean gridColumn = (x % doubledGridSize) == 0;
                boolean gridRow = (y % doubledGridSize) == 0;
                // compute color for each pixel
                int color = gridColumn || gridRow ? 0 : -1;
                gridTexture.setRGB(x, y, color);
            }
        }
        return gridTexture;
    }

    /**
     * Create diagonal grid texture where tile sizes are based on grid size.
     * Grid angle is set to 45 degrees.
     * 
     * @param gridSize
     *            size of a grid
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @return buffered image containing grid texture
     */
    private static BufferedImage diagonalGridTextureFactory(int gridSize, int width, int height)
    {
        // used by test for a tile colors
        int doubledGridSize = gridSize << 1;

        // if the grid texture already exists, return it
        if (diagonalGridTexture != null)
        {
            return diagonalGridTexture;
        }

        // create new texture instead
        diagonalGridTexture = new BufferedImage(width, height, BufferedImage.TYPE_BYTE_BINARY);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each tile
                boolean gridColumn = ((x+y) % doubledGridSize) == 0;
                boolean gridRow = ((x-y) % doubledGridSize) == 0;
                // compute color for each pixel
                int color = gridColumn || gridRow ? 0 : -1;
                diagonalGridTexture.setRGB(x, y, color);
            }
        }
        return diagonalGridTexture;
    }

    /**
     * Create texture containing horizontal stripes.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @return buffered image containing texture
     */
    private static BufferedImage horizontalStripesTextureFactory(int width, int height)
    {
        // if the texture already exists, return it
        if (horizontalStripesTexture != null)
        {
            return horizontalStripesTexture;
        }

        // create new texture instead
        horizontalStripesTexture  = new BufferedImage(width, height, BufferedImage.TYPE_BYTE_BINARY);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                int color = (y % 2) == 0 ? 0 : -1;
                horizontalStripesTexture.setRGB(x, y, color);
            }
        }
        return horizontalStripesTexture;
    }

    /**
     * Create texture containing vertical stripes.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @return buffered image containing texture
     */
    private static BufferedImage verticalStripesTextureFactory(int width, int height)
    {
        // if the texture already exists, return it
        if (verticalStripesTexture != null)
        {
            return verticalStripesTexture;
        }

        // create new texture instead
        verticalStripesTexture  = new BufferedImage(width, height, BufferedImage.TYPE_BYTE_BINARY);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                int color = (x % 2) == 0 ? 0 : -1;
                verticalStripesTexture.setRGB(x, y, color);
            }
        }
        return verticalStripesTexture;
    }

    /**
     * Create texture containing horizontal color stripes containing 8 basic colors.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @return buffered image containing texture
     */
    private static BufferedImage horizontalColorStripesTextureFactory(int width, int height)
    {
        // if the texture already exists, return it
        if (horizontalColorStripesTexture != null)
        {
            return horizontalColorStripesTexture;
        }

        // create new texture instead
        horizontalColorStripesTexture  = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // compute color for each pixel
            int color = EIGHT_BASIC_COLOR_PALETTE[y % 8];
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                horizontalColorStripesTexture.setRGB(x, y, color);
            }
        }
        return horizontalColorStripesTexture;
    }

    /**
     * Create texture containing vertical stripes containing 8 basic colors.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @return buffered image containing texture
     */
    private static BufferedImage verticalColorStripesTextureFactory(int width, int height)
    {
        // if the texture already exists, return it
        if (verticalColorStripesTexture != null)
        {
            return verticalColorStripesTexture;
        }

        // create new texture instead
        verticalColorStripesTexture  = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                int color = EIGHT_BASIC_COLOR_PALETTE[x % 8];
                verticalColorStripesTexture.setRGB(x, y, color);
            }
        }
        return verticalColorStripesTexture;
    }

    /**
     * Create texture containing diagonal stripes.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @return buffered image containing texture
     */
    private static BufferedImage diagonalStripesTextureFactory(int width, int height)
    {
        // if the texture already exists, return it
        if (diagonalStripesTexture != null)
        {
            return diagonalStripesTexture;
        }

        // create new texture instead
        diagonalStripesTexture  = new BufferedImage(width, height, BufferedImage.TYPE_BYTE_BINARY);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                int color = (x+y) % 2 == 0 ? 0 : -1;
                diagonalStripesTexture.setRGB(x, y, color);
            }
        }
        return diagonalStripesTexture;
    }

    /**
     * Create texture containing color dots.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @return buffered image containing texture
     */
    private static BufferedImage colorDotsTextureFactory(int width, int height)
    {
        // if the texture already exists, return it
        if (colorDotsTexture != null)
        {
            return colorDotsTexture;
        }

        // create new texture instead
        colorDotsTexture  = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                int color;
                // set color to white on even row and column 
                if (x % 2 == 0 || y % 2 == 0)
                {
                    color = -1;
                }
                else
                {
                    int red = x % 4 == 1 ? 0x00 : 0xff;
                    int green = y % 4 == 1 ? 0x00 : 0xff;
                    int blue = 0;
                    color = makeRGBColor(red, green, blue);
                }
                colorDotsTexture.setRGB(x, y, color);
            }
        }
        return colorDotsTexture;
    }

    /**
     * Create first version of RGB texture.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @return buffered image containing texture
     */
    private static BufferedImage RGBTexture1Factory(int width, int height)
    {
        // if the texture already exists, return it
        if (RGBTexture1 != null)
        {
            return RGBTexture1;
        }

        // create new texture instead
        RGBTexture1  = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                int red = colorComponentFromXOrYCoordinate(width, x);
                int green = colorComponentFromXOrYCoordinate(height, y);
                int blue = 0;
                int color = makeRGBColor(red, green, blue);
                RGBTexture1.setRGB(x, y, color);
            }
        }
        return RGBTexture1;
    }

    /**
     * Create second version of RGB texture.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @return buffered image containing texture
     */
    private static BufferedImage RGBTexture2Factory(int width, int height)
    {
        // if the texture already exists, return it
        if (RGBTexture2 != null)
        {
            return RGBTexture2;
        }

        // create new texture instead
        RGBTexture2  = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                int red = colorComponentFromXOrYCoordinate(width, x);
                int green = colorComponentFromXOrYCoordinate(height, y);
                int blue = 0xff;
                int color = makeRGBColor(red, green, blue);
                RGBTexture2.setRGB(x, y, color);
            }
        }
        return RGBTexture2;
    }

    /**
     * Create third version of RGB texture.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @return buffered image containing texture
     */
    private static BufferedImage RGBTexture3Factory(int width, int height)
    {
        // if the texture already exists, return it
        if (RGBTexture3 != null)
        {
            return RGBTexture3;
        }

        // create new texture instead
        RGBTexture3  = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                int red = 0;
                int green = colorComponentFromXOrYCoordinate(height, y);
                int blue = colorComponentFromXOrYCoordinate(width, x);
                int color = makeRGBColor(red, green, blue);
                RGBTexture3.setRGB(x, y, color);
            }
        }
        return RGBTexture3;
    }

    /**
     * Create fourth version of RGB texture.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @return buffered image containing texture
     */
    private static BufferedImage RGBTexture4Factory(int width, int height)
    {
        // if the texture already exists, return it
        if (RGBTexture4 != null)
        {
            return RGBTexture4;
        }

        // create new texture instead
        RGBTexture4  = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                int red = 0xff;
                int green = colorComponentFromXOrYCoordinate(width, x);
                int blue = colorComponentFromXOrYCoordinate(height, y);
                int color = makeRGBColor(red, green, blue);
                RGBTexture4.setRGB(x, y, color);
            }
        }
        return RGBTexture4;
    }

    /**
     * Create fifth version of RGB texture.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @return buffered image containing texture
     */
    private static BufferedImage RGBTexture5Factory(int width, int height)
    {
        // if the texture already exists, return it
        if (RGBTexture5 != null)
        {
            return RGBTexture5;
        }

        // create new texture instead
        RGBTexture5  = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                int red = colorComponentFromXOrYCoordinate(width, x);
                int green = 0;
                int blue = colorComponentFromXOrYCoordinate(height, y);
                int color = makeRGBColor(red, green, blue);
                RGBTexture5.setRGB(x, y, color);
            }
        }
        return RGBTexture5;
    }


    /**
     * Create sixth version of RGB texture.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @return buffered image containing texture
     */
    private static BufferedImage RGBTexture6Factory(int width, int height)
    {
        // if the texture already exists, return it
        if (RGBTexture6 != null)
        {
            return RGBTexture6;
        }

        // create new texture instead
        RGBTexture6  = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                int red = colorComponentFromXOrYCoordinate(width, x);
                int green = 0xff;
                int blue = colorComponentFromXOrYCoordinate(height, y);
                int color = makeRGBColor(red, green, blue);
                RGBTexture6.setRGB(x, y, color);
            }
        }
        return RGBTexture6;
    }

    /**
     * @param red
     * @param green
     * @param blue
     * @return
     */
    private static int makeRGBColor(int red, int green, int blue)
    {
        return (0xff << 24) | (red << 16) | (green << 8) | blue;
    }

    /**
     * @param width
     * @param x
     * @return
     */
    private static int colorComponentFromXOrYCoordinate(int width, int x)
    {
        return (x - (width >> 1)) & 0xff;
    }

    /**
     * Return {@link TexturePaint} object containing checker texture.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param anchor
     *            the Rectangle2D in user space used to anchor and replicate the
     *            texture
     * @return {@link TexturePaint} object containing checker texture
     */
    public static TexturePaint getCheckerTexturePaint(TestImage image, Rectangle2D anchor)
    {
        return new TexturePaint(checkerTextureFactory(image.getGrid(), image.getWidth(), image.getHeight()), anchor);
    }

    /**
     * Return {@link TexturePaint} object containing diagonal checker texture.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param anchor
     *            the Rectangle2D in user space used to anchor and replicate the
     *            texture
     * @return {@link TexturePaint} object containing checker texture
     */
    public static TexturePaint getDiagonalCheckerTexturePaint(TestImage image, Rectangle2D anchor)
    {
        return new TexturePaint(diagonalCheckerTextureFactory(image.getGrid(), image.getWidth(), image.getHeight()), anchor);
    }

    /**
     * Return {@link TexturePaint} object containing grid texture.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param anchor
     *            the Rectangle2D in user space used to anchor and replicate the
     *            texture
     * @return {@link TexturePaint} object containing grid texture
     */
    public static TexturePaint getGridTexturePaint(TestImage image, Rectangle2D anchor)
    {
        return new TexturePaint(gridTextureFactory(image.getGrid(), image.getWidth(), image.getHeight()), anchor);
    }

    /**
     * Return {@link TexturePaint} object containing diagonal grid texture.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param anchor
     *            the Rectangle2D in user space used to anchor and replicate the
     *            texture
     * @return {@link TexturePaint} object containing diagonal grid texture
     */
    public static TexturePaint getDiagonalGridTexturePaint(TestImage image, Rectangle2D anchor)
    {
        return new TexturePaint(diagonalGridTextureFactory(image.getGrid(), image.getWidth(), image.getHeight()), anchor);
    }

    /**
     * Return {@link TexturePaint} object containing texture with horizontal stripes.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param anchor
     *            the Rectangle2D in user space used to anchor and replicate the
     *            texture
     * @return {@link TexturePaint} object containing horizontal stripes texture
     */
    public static TexturePaint getHorizontalStripesPaint(TestImage image, Rectangle2D anchor)
    {
        return new TexturePaint(horizontalStripesTextureFactory(image.getWidth(), image.getHeight()), anchor);
    }

    /**
     * Return {@link TexturePaint} object containing texture with vertical stripes.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param anchor
     *            the Rectangle2D in user space used to anchor and replicate the
     *            texture
     * @return {@link TexturePaint} object containing vertical stripes texture
     */
    public static TexturePaint getVerticalStripesPaint(TestImage image, Rectangle2D anchor)
    {
        return new TexturePaint(verticalStripesTextureFactory(image.getWidth(), image.getHeight()), anchor);
    }

    /**
     * Return {@link TexturePaint} object containing texture with horizontal color stripes.
     * These stripes contain eight basic colors.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param anchor
     *            the Rectangle2D in user space used to anchor and replicate the
     *            texture
     * @return {@link TexturePaint} object containing horizontal stripes texture
     */
    public static TexturePaint getHorizontalColorStripesPaint(TestImage image, Rectangle2D anchor)
    {
        return new TexturePaint(horizontalColorStripesTextureFactory(image.getWidth(), image.getHeight()), anchor);
    }

    /**
     * Return {@link TexturePaint} object containing texture with vertical color stripes.
     * These stripes contain eight basic colors.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param anchor
     *            the Rectangle2D in user space used to anchor and replicate the
     *            texture
     * @return {@link TexturePaint} object containing vertical stripes texture
     */
    public static TexturePaint getVerticalColorStripesPaint(TestImage image, Rectangle2D anchor)
    {
        return new TexturePaint(verticalColorStripesTextureFactory(image.getWidth(), image.getHeight()), anchor);
    }

    /**
     * Return {@link TexturePaint} object containing texture with diagonal stripes.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param anchor
     *            the Rectangle2D in user space used to anchor and replicate the
     *            texture
     * @return {@link TexturePaint} object containing diagonal stripes texture
     */
    public static TexturePaint getDiagonalStripesPaint(TestImage image, Rectangle2D anchor)
    {
        return new TexturePaint(diagonalStripesTextureFactory(image.getWidth(), image.getHeight()), anchor);
    }

    /**
     * Return {@link TexturePaint} object containing texture with color dots.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param anchor
     *            the Rectangle2D in user space used to anchor and replicate the
     *            texture
     * @return {@link TexturePaint} object containing diagonal stripes texture
     */
    public static TexturePaint getColorDotsPaint(TestImage image, Rectangle2D anchor)
    {
        return new TexturePaint(colorDotsTextureFactory(image.getWidth(), image.getHeight()), anchor);
    }

    /**
     * Return {@link TexturePaint} object containing texture with various RGB colors.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param anchor
     *            the Rectangle2D in user space used to anchor and replicate the
     *            texture
     * @return {@link TexturePaint} object containing gradient texture
     */
    public static TexturePaint getRGBTexture1Paint(TestImage image, Rectangle2D anchor)
    {
        return new TexturePaint(RGBTexture1Factory(image.getWidth(), image.getHeight()), anchor);
    }

    /**
     * Return {@link TexturePaint} object containing texture with various RGB colors.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param anchor
     *            the Rectangle2D in user space used to anchor and replicate the
     *            texture
     * @return {@link TexturePaint} object containing gradient texture
     */
    public static TexturePaint getRGBTexture2Paint(TestImage image, Rectangle2D anchor)
    {
        return new TexturePaint(RGBTexture2Factory(image.getWidth(), image.getHeight()), anchor);
    }

    /**
     * Return {@link TexturePaint} object containing texture with various RGB colors.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param anchor
     *            the Rectangle2D in user space used to anchor and replicate the
     *            texture
     * @return {@link TexturePaint} object containing gradient texture
     */
    public static TexturePaint getRGBTexture3Paint(TestImage image, Rectangle2D anchor)
    {
        return new TexturePaint(RGBTexture3Factory(image.getWidth(), image.getHeight()), anchor);
    }

    /**
     * Return {@link TexturePaint} object containing texture with various RGB colors.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param anchor
     *            the Rectangle2D in user space used to anchor and replicate the
     *            texture
     * @return {@link TexturePaint} object containing gradient texture
     */
    public static TexturePaint getRGBTexture4Paint(TestImage image, Rectangle2D anchor)
    {
        return new TexturePaint(RGBTexture4Factory(image.getWidth(), image.getHeight()), anchor);
    }

    /**
     * Return {@link TexturePaint} object containing texture with various RGB colors.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param anchor
     *            the Rectangle2D in user space used to anchor and replicate the
     *            texture
     * @return {@link TexturePaint} object containing gradient texture
     */
    public static TexturePaint getRGBTexture5Paint(TestImage image, Rectangle2D anchor)
    {
        return new TexturePaint(RGBTexture5Factory(image.getWidth(), image.getHeight()), anchor);
    }

    /**
     * Return {@link TexturePaint} object containing texture with various RGB colors.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param anchor
     *            the Rectangle2D in user space used to anchor and replicate the
     *            texture
     * @return {@link TexturePaint} object containing gradient texture
     */
    public static TexturePaint getRGBTexture6Paint(TestImage image, Rectangle2D anchor)
    {
        return new TexturePaint(RGBTexture6Factory(image.getWidth(), image.getHeight()), anchor);
    }

    /**
     * Create new RGB pattern using given image type.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @param imageType
     *            required image type
     * @return buffered image containing texture
     */
    public static BufferedImage createRGBTexture1(int width, int height, int imageType)
    {
        // create new texture instead
        RGBTexture1  = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                int red = colorComponentFromXOrYCoordinate(width, x);
                int green = colorComponentFromXOrYCoordinate(height, y);
                int blue = 0;
                int color = makeRGBColor(red, green, blue);
                RGBTexture1.setRGB(x, y, color);
            }
        }
        return RGBTexture1;
    }

    /**
     * Create new RGB pattern using given image type.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @param imageType
     *            required image type
     * @return buffered image containing texture
     */
    public static BufferedImage createRGBTexture2(int width, int height, int imageType)
    {
        // create new texture instead
        RGBTexture2  = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                int red = colorComponentFromXOrYCoordinate(width, x);
                int green = colorComponentFromXOrYCoordinate(height, y);
                int blue = 0xff;
                int color = makeRGBColor(red, green, blue);
                RGBTexture2.setRGB(x, y, color);
            }
        }
        return RGBTexture2;
    }

    /**
     * Create new RGB pattern using given image type.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @param imageType
     *            required image type
     * @return buffered image containing texture
     */
    public static BufferedImage createRGBTexture3(int width, int height, int imageType)
    {
        // create new texture instead
        RGBTexture3  = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                int red = 0;
                int green = colorComponentFromXOrYCoordinate(height, y);
                int blue = colorComponentFromXOrYCoordinate(width, x);
                int color = makeRGBColor(red, green, blue);
                RGBTexture3.setRGB(x, y, color);
            }
        }
        return RGBTexture3;
    }

    /**
     * Create new RGB pattern using given image type.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @param imageType
     *            required image type
     * @return buffered image containing texture
     */
    public static BufferedImage createRGBTexture4(int width, int height, int imageType)
    {
        // create new texture instead
        RGBTexture4  = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                int red = 0xff;
                int green = colorComponentFromXOrYCoordinate(width, x);
                int blue = colorComponentFromXOrYCoordinate(height, y);
                int color = makeRGBColor(red, green, blue);
                RGBTexture4.setRGB(x, y, color);
            }
        }
        return RGBTexture4;
    }

    /**
     * Create new RGB pattern using given image type.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @param imageType
     *            required image type
     * @return buffered image containing texture
     */
    public static BufferedImage createRGBTexture5(int width, int height, int imageType)
    {
        // create new texture instead
        RGBTexture5  = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                int red = colorComponentFromXOrYCoordinate(width, x);
                int green = 0;
                int blue = colorComponentFromXOrYCoordinate(height, y);
                int color = makeRGBColor(red, green, blue);
                RGBTexture5.setRGB(x, y, color);
            }
        }
        return RGBTexture5;
    }

    /**
     * Create new RGB pattern using given image type.
     * 
     * @param width
     *            width of a texture
     * @param height
     *            height of a texture
     * @param imageType
     *            required image type
     * @return buffered image containing texture
     */
    public static BufferedImage createRGBTexture6(int width, int height, int imageType)
    {
        // create new texture instead
        RGBTexture6  = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                int red = colorComponentFromXOrYCoordinate(width, x);
                int green = 0xff;
                int blue = colorComponentFromXOrYCoordinate(height, y);
                int color = makeRGBColor(red, green, blue);
                RGBTexture6.setRGB(x, y, color);
            }
        }
        return RGBTexture6;
    }

}
