/*
  Java gfx-test framework

   Copyright (C) 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.testsuites;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.geom.Path2D;



import org.gfxtest.callbacks.CubicCurveDrawCallbacks;
import org.gfxtest.framework.ColorPalette;
import org.gfxtest.framework.GrayscalePalette;
import org.gfxtest.framework.HSBPalette;
import org.gfxtest.framework.PrintTest;
import org.gfxtest.framework.TestImage;
import org.gfxtest.framework.TestResult;
import org.gfxtest.framework.annotations.GraphicsPrimitive;
import org.gfxtest.framework.annotations.GraphicsPrimitives;
import org.gfxtest.framework.annotations.PathVariant;
import org.gfxtest.framework.annotations.PathVariants;
import org.gfxtest.framework.annotations.RenderStyle;
import org.gfxtest.framework.annotations.RenderStyles;
import org.gfxtest.framework.annotations.TestType;
import org.gfxtest.framework.annotations.TestTypes;
import org.gfxtest.framework.annotations.Transformation;
import org.gfxtest.framework.annotations.Transformations;
import org.gfxtest.framework.annotations.Zoom;



/**
 * This test suite checks rendering of various cubic curves. Cubic curves are
 * rendered as paths which consists of only one segment - the curve itself.
 * 
 * @author Pavel Tisnovsky
 */
@TestType(TestTypes.PRINT_TEST)
@GraphicsPrimitive(GraphicsPrimitives.PATH)
@PathVariant(PathVariants.CUBIC_CURVE)
@RenderStyle(RenderStyles.NORMAL)
@Transformation(Transformations.NONE)
@Zoom(1)
public class PrintTestCubicCurvesAsPaths extends PrintTest
{

    /**
     * Create Path2D which contains only one cubic curve.
     * 
     * @param x1
     *            x coordinate of the first control point
     * @param y1
     *            y coordinate of the first control point
     * @param x2
     *            x coordinate of the second control point
     * @param y2
     *            y coordinate of the second control point
     * @param x3
     *            x coordinate of third control point
     * @param y3
     *            y coordinate of third control point
     * @param x4
     *            x coordinate of fourth control point
     * @param y4
     *            y coordinate of fourth control point
     * @return newly created path
     */
    private static Path2D createCubicCurvePath(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4)
    {
        Path2D path = new Path2D.Float();
        // first control point
        path.moveTo(x1, y1);
        // second, third and fourth control point
        path.curveTo(x2, y2, x3, y3, x4, y4);
        return path;
    }

    /**
     * Method which renders set of cubic curves using various colors and
     * stroke styles. For each curve, the callback function/method is called to
     * perform all required setup. Curves are represented by a Path object.
     * 
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @param verticalStep
     *            between two near curves
     * @param cubicCurveCallback
     *            class containing set of callback methods
     */
    private static void drawCubicCurve(TestImage image, Graphics2D graphics, int verticalStep, CubicCurveDrawCallbacks cubicCurveCallback)
    {
        // setup rendering
        cubicCurveCallback.setup(image, graphics);

        // image width and height
        final int width = image.getWidth();
        final int height = image.getHeight();

        // horizontal coordinates of curve endpoints
        final int x1 = BORDER;
        final int x4 = width - BORDER;
        final int x2 = (x1 + x4) * 1 / 3;
        final int x3 = (x1 + x4) * 2 / 3;

        // index to color palette
        int colorIndex = 0;

        // draw all lines onto a paper
        for (int y = BORDER * 10; y < height - BORDER * 10; y += verticalStep)
        {
            // compute other control points coordinates
            final int y1 = y;
            final int y2 = y - (BORDER << 4);
            final int y3 = y + (BORDER << 4);
            final int y4 = y;
            // setup can be made for each line
            cubicCurveCallback.iterationCallBack(x1, y1, x2, y2, x3, y3, x4, y4, colorIndex++);
            // create new path which contains just one line
            Path2D path = createCubicCurvePath(x1, y1, x2, y2, x3, y3, x4, y4);
            // render the line
            graphics.draw(path);
        }

        // cleanup rendering
        cubicCurveCallback.cleanup();
    }

    /**
     * Test basic behavior of method Graphics.draw(Path).
     * Cubic curves are rendered with default width and default end caps.
     * Color of all rendered curves are set to black.
     * All rendered cubic curves are represented by a Path object.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCubicCurvesBasicStyle(TestImage image, Graphics2D graphics)
    {
        drawCubicCurve(image, graphics, CUBIC_CURVE_STEP, new CubicCurveDrawCallbacks()
        {
            /**
             * Callback function called before each cubic curve is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int index)
            {
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.draw(Path).
     * Cubic curves are rendered with default width and default end caps.
     * Color of all rendered curves are selected from a palette.
     * All rendered cubic curves are represented by a Path object.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCubicCurvesColorPalette(TestImage image, Graphics2D graphics)
    {
        drawCubicCurve(image, graphics, CUBIC_CURVE_STEP, new CubicCurveDrawCallbacks()
        {
            /**
             * Callback function called before each cubic curve is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int index)
            {
                // set curve color
                this.graphics.setColor(ColorPalette.getColor(index));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.draw(Path).
     * Cubic curves are rendered with default width and default end caps.
     * Color of all rendered curves are selected from a grayscale palette.
     * All rendered cubic curves are represented by a Path object.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCubicCurvesGrayScale(TestImage image, Graphics2D graphics)
    {
        drawCubicCurve(image, graphics, CUBIC_CURVE_STEP, new CubicCurveDrawCallbacks()
        {
            /**
             * Callback function called before each cubic curve is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int index)
            {
                // compute grayscale value
                float gray = y1 * 1.0f / this.image.getHeight();
                // set curve color
                this.graphics.setColor(GrayscalePalette.createGrayscaleColor(gray));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.draw(Path).
     * Cubic curves are rendered with default width and default end caps.
     * All rendered cubic curves are represented by a Path object.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCubicCurvesColorScale(TestImage image, Graphics2D graphics)
    {
        drawCubicCurve(image, graphics, CUBIC_CURVE_STEP, new CubicCurveDrawCallbacks()
        {
            /**
             * Callback function called before each cubic curve is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int index)
            {
                // compute hue value
                float hue = y1 * 1.0f / this.image.getHeight();
                // set curve color
                this.graphics.setColor(HSBPalette.createHsbColor(hue));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.draw(Path).
     * Cubic curves are rendered with various width and default end caps.
     * Color of all rendered curves are selected from a grayscale palette.
     * All rendered cubic curves are represented by a Path object.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCubicCurvesChangeWidth(TestImage image, Graphics2D graphics)
    {
        drawCubicCurve(image, graphics, CUBIC_CURVE_STEP, new CubicCurveDrawCallbacks()
        {
            /**
             * Stroke width.
             */
            float strokeWidth = 0.0f;

            /**
             * Callback function called before each cubic curve is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int index)
            {
                // set stroke width
                this.graphics.setStroke(new BasicStroke(this.strokeWidth));
                // set new stroke width
                this.strokeWidth = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.draw(Path).
     * Cubic curves are rendered with various width and end caps set to CAP_BUTT.
     * Join style is set to bevel style.
     * Color of all rendered curves are selected from a grayscale palette.
     * All rendered cubic curves are represented by a Path object.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCubicCurvesChangeWidthCapButtJoinBevel(TestImage image, Graphics2D graphics)
    {
        drawCubicCurve(image, graphics, CUBIC_CURVE_STEP, new CubicCurveDrawCallbacks()
        {
            /**
             * Stroke width.
             */
            float strokeWidth = 0.0f;

            /**
             * Callback function called before each cubic curve is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int index)
            {
                // set stroke width
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL));
                // set new stroke width
                this.strokeWidth = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.draw(Path).
     * Cubic curves are rendered with various width and end caps set to CAP_ROUND.
     * Join style is set to bevel style.
     * Color of all rendered curves are selected from a grayscale palette.
     * All rendered cubic curves are represented by a Path object.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCubicCurvesChangeWidthCapRoundJoinBevel(TestImage image, Graphics2D graphics)
    {
        drawCubicCurve(image, graphics, CUBIC_CURVE_STEP, new CubicCurveDrawCallbacks()
        {
            /**
             * Stroke width.
             */
            float strokeWidth = 0.0f;

            /**
             * Callback function called before each cubic curve is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int index)
            {
                // set stroke width
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL));
                // set new stroke width
                this.strokeWidth = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.draw(Path).
     * Cubic curves are rendered with various width and end caps set to CAP_SQUARE.
     * Join style is set to bevel style.
     * Color of all rendered curves are selected from a grayscale palette.
     * All rendered cubic curves are represented by a Path object.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCubicCurvesChangeWidthCapSquareJoinBevel(TestImage image, Graphics2D graphics)
    {
        drawCubicCurve(image, graphics, CUBIC_CURVE_STEP, new CubicCurveDrawCallbacks()
        {
            /**
             * Stroke width.
             */
            float strokeWidth = 0.0f;

            /**
             * Callback function called before each cubic curve is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int index)
            {
                // set stroke width
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL));
                // set new stroke width
                this.strokeWidth = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.draw(Path).
     * Cubic curves are rendered with various width and end caps set to CAP_BUTT.
     * Join style is set to miter style.
     * Color of all rendered curves are selected from a grayscale palette.
     * All rendered cubic curves are represented by a Path object.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCubicCurvesChangeWidthCapButtJoinMiter(TestImage image, Graphics2D graphics)
    {
        drawCubicCurve(image, graphics, CUBIC_CURVE_STEP, new CubicCurveDrawCallbacks()
        {
            /**
             * Stroke width.
             */
            float strokeWidth = 0.0f;

            /**
             * Callback function called before each cubic curve is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int index)
            {
                // set stroke width
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER));
                // set new stroke width
                this.strokeWidth = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.draw(Path).
     * Cubic curves are rendered with various width and end caps set to CAP_ROUND.
     * Join style is set to miter style.
     * Color of all rendered curves are selected from a grayscale palette.
     * All rendered cubic curves are represented by a Path object.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCubicCurvesChangeWidthCapRoundJoinMiter(TestImage image, Graphics2D graphics)
    {
        drawCubicCurve(image, graphics, CUBIC_CURVE_STEP, new CubicCurveDrawCallbacks()
        {
            /**
             * Stroke width.
             */
            float strokeWidth = 0.0f;

            /**
             * Callback function called before each cubic curve is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int index)
            {
                // set stroke width
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                // set new stroke width
                this.strokeWidth = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.draw(Path).
     * Cubic curves are rendered with various width and end caps set to CAP_SQUARE.
     * Join style is set to miter style.
     * Color of all rendered curves are selected from a grayscale palette.
     * All rendered cubic curves are represented by a Path object.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCubicCurvesChangeWidthCapSquareJoinMiter(TestImage image, Graphics2D graphics)
    {
        drawCubicCurve(image, graphics, CUBIC_CURVE_STEP, new CubicCurveDrawCallbacks()
        {
            /**
             * Stroke width.
             */
            float strokeWidth = 0.0f;

            /**
             * Callback function called before each cubic curve is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int index)
            {
                // set stroke width
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER));
                // set new stroke width
                this.strokeWidth = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }


    /**
     * Test basic behavior of method Graphics.draw(Path).
     * Cubic curves are rendered with various width and end caps set to CAP_BUTT.
     * Join style is set to round style.
     * Color of all rendered curves are selected from a grayscale palette.
     * All rendered cubic curves are represented by a Path object.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCubicCurvesChangeWidthCapButtJoinRound(TestImage image, Graphics2D graphics)
    {
        drawCubicCurve(image, graphics, CUBIC_CURVE_STEP, new CubicCurveDrawCallbacks()
        {
            /**
             * Stroke width.
             */
            float strokeWidth = 0.0f;

            /**
             * Callback function called before each cubic curve is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int index)
            {
                // set stroke width
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_ROUND));
                // set new stroke width
                this.strokeWidth = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.draw(Path).
     * Cubic curves are rendered with various width and end caps set to CAP_ROUND.
     * Join style is set to round style.
     * Color of all rendered curves are selected from a grayscale palette.
     * All rendered cubic curves are represented by a Path object.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCubicCurvesChangeWidthCapRoundJoinRound(TestImage image, Graphics2D graphics)
    {
        drawCubicCurve(image, graphics, CUBIC_CURVE_STEP, new CubicCurveDrawCallbacks()
        {
            /**
             * Stroke width.
             */
            float strokeWidth = 0.0f;

            /**
             * Callback function called before each cubic curve is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int index)
            {
                // set stroke width
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
                // set new stroke width
                this.strokeWidth = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.draw(Path).
     * Cubic curves are rendered with various width and end caps set to CAP_SQUARE.
     * Join style is set to round style.
     * Color of all rendered curves are selected from a grayscale palette.
     * All rendered cubic curves are represented by a Path object.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCubicCurvesChangeWidthCapSquareJoinRound(TestImage image, Graphics2D graphics)
    {
        drawCubicCurve(image, graphics, CUBIC_CURVE_STEP, new CubicCurveDrawCallbacks()
        {
            /**
             * Stroke width.
             */
            float strokeWidth = 0.0f;

            /**
             * Callback function called before each cubic curve is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int index)
            {
                // set stroke width
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_ROUND));
                // set new stroke width
                this.strokeWidth = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Entry point to the test suite.
     * 
     * @param args
     *            not used in this case
     */
    public static void main(String[] args)
    {
        new PrintTestCubicCurvesAsPaths().runTestSuite(args);
    }
}
