/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.testsuites;

import java.awt.*;
import java.awt.geom.Path2D;

import org.gfxtest.framework.*;
import org.gfxtest.framework.annotations.*;



/**
 * This test renders various special cases, such as curves with cusps, curves
 * with identical control points, curves with width such that the offset curves
 * have cusps, curves with very high curvatures, paths such as moveTo(0, 0);
 * p.lineTo(100,100);p.lineTo(100,100); (we test this last one to make sure that
 * the join is drawn even though the lines are at an angle of 180 degrees. For
 * each path, we draw it, and we also draw its stroked version.
 * 
 * @author Pavel Tisnovsky
 * @author Denis Lila
 */
@TestType(TestTypes.RENDER_TEST)
@RenderStyle(RenderStyles.NORMAL)
@Transformation(Transformations.NONE)
@Zoom(1)
public class SpecialCases extends GfxTest
{
    /**
     * Draw path onto the specified graphics context. Path is drawn as the path
     * itself and also as outline.
     * 
     * @param path
     *            path to be drawn
     * @param pw
     *            width of the path
     * @param graphics
     *            graphics context for image
     * @param width
     *            width of the canvas
     */
    private void drawPath(Path2D path, float pw, Graphics2D graphics, int width)
    {
        BasicStroke basicStroke = new BasicStroke(pw, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_ROUND);
        graphics.setColor(java.awt.Color.black);
        Shape shape = basicStroke.createStrokedShape(path);
        graphics.draw(shape);
        graphics.translate(width, 0);
        Stroke orig = graphics.getStroke();
        graphics.setStroke(basicStroke);
        graphics.draw(path);
        graphics.setStroke(orig);
        graphics.translate(-width, 0);
    }

    /**
     * Test drawing of line segments with identical vertexes.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalVertexesV(TestImage image, Graphics2D graphics)
    {
        int width = image.getWidth() / 2;
        int height = image.getHeight();
        int size = Math.min(width, height);
        Path2D path = new Path2D.Double();
        for (double d = 0.1; d < 1.0; d += 0.2)
        {
            path.moveTo(d * size, 0.1 * size);
            path.lineTo(d * size, d * size);
        }
        drawPath(path, 0.15f * size, graphics, width);
        return TestResult.PASSED;
    }

    /**
     * Test drawing of line segments with identical vertexes.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalVertexesH(TestImage image, Graphics2D graphics)
    {
        int width = image.getWidth() / 2;
        int height = image.getHeight();
        int size = Math.min(width, height);
        Path2D path = new Path2D.Double();
        for (double d = 0.1; d < 1.0; d += 0.2)
        {
            path.moveTo(0.1 * size, d * size);
            path.lineTo(d * size, d * size);
        }
        drawPath(path, 0.15f * size, graphics, width);
        return TestResult.PASSED;
    }

    /**
     * Test drawing of Bezier cubic curves with identical control points.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalCtrlCub(TestImage image, Graphics2D graphics)
    {
        int width = image.getWidth() / 2;
        int height = image.getHeight();
        int size = Math.min(width, height);
        Path2D path = new Path2D.Double();
        path.moveTo(0, 0);
        // identical first 2 points
        path.curveTo(0, 0, 0.1 * size, 0.2 * size, 0.2 * size, 0.2 * size);
        // identical 2nd and 3rd points
        path.curveTo(0.35 * size, 0.8 * size, 0.35 * size, 0.8 * size, 0.45 * size, 0.1 * size);
        // identical 3rd and 4th points.
        path.curveTo(0.60 * size, 0.1 * size, 0.60 * size, 0.70 * size, 0.60 * size, 0.70 * size);
        // identical first 3 points
        path.curveTo(0.6 * size, 0.7 * size, 0.6 * size, 0.7 * size, 0.5 * size, 0.8 * size);
        // identical last 3 points
        path.curveTo(0.3 * size, 0.8 * size, 0.3 * size, 0.8 * size, 0.3 * size, 0.8 * size);
        // all points identical
        path.moveTo(0.1 * size, 0.8 * size);
        path.curveTo(0.1 * size, 0.8 * size, 0.1 * size, 0.8 * size, 0.1 * size, 0.8 * size);
        drawPath(path, 0.15f * size, graphics, width);
        return TestResult.PASSED;
    }

    /**
     * Test drawing of Bezier quadratic curves with identical control points.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalCtrlQuad(TestImage image, Graphics2D graphics)
    {
        int width = image.getWidth() / 2;
        int height = image.getHeight();
        int size = Math.min(width, height);
        Path2D path = new Path2D.Double();
        path.moveTo(0, 0);
        // identical first 2 points
        path.quadTo(0, 0, 0.1 * size, 0.2 * size);
        // identical 2nd and 3rd points
        path.quadTo(0.7 * size, 0.6 * size, 0.7 * size, 0.6 * size);
        // no identical points. We just want to see what a quad looks like.
        path.quadTo(1.0 * size, 0.7 * size, 0.1 * size, 0.8 * size);
        // all points identical
        path.moveTo(0.7 * size, 0.3 * size);
        path.quadTo(0.7 * size, 0.3 * size, 0.7 * size, 0.3 * size);
        drawPath(path, 0.2f * size, graphics, width);
        return TestResult.PASSED;
    }

    /**
     * Test drawing of a path constructed from two line segments which have the
     * same vertexes.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult test180Turn(TestImage image, Graphics2D graphics)
    {
        int width = image.getWidth() / 2;
        int height = image.getHeight();
        int size = Math.min(width, height);
        int numlines = 6;
        double cx = 0.5 * size, cy = 0.5 * size;
        for (int i = 0; i < numlines; i++)
        {
            double dx = 0.4 * size * Math.cos((i * 2 * Math.PI) / numlines);
            double dy = 0.4 * size * Math.sin((i * 2 * Math.PI) / numlines);
            Path2D path = new Path2D.Double();
            path.moveTo(cx, cy);
            path.lineTo(cx + dx, cy + dy);
            path.lineTo(cx, cy);
            drawPath(path, (float) Math.pow(0.05 * size * (i + 1), 0.8), graphics, width);
        }
        return TestResult.PASSED;
    }

    /**
     * Test drawing closed path constructed from line segments.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCloseLineSegments(TestImage image, Graphics2D graphics)
    {
        int width = image.getWidth() / 2;
        int height = image.getHeight();
        int size = Math.min(width, height);
        Path2D path = new Path2D.Double();
        path.moveTo(0.1 * size, 0.1 * size);
        path.lineTo(0.8 * size, 0.4 * size);
        path.lineTo(0.55 * size, 1.0 * size);
        path.lineTo(0.6 * size, 0.5 * size);
        path.lineTo(0.1 * size, 0.3 * size);
        path.lineTo(0.0 * size, 0.9 * size);
        path.lineTo(0.65 * size, 0.9 * size);
        path.closePath();
        drawPath(path, 0.1f * size, graphics, width);
        return TestResult.PASSED;
    }

    /**
     * Test drawing closed path constructed from quadratic Bezier curve segments.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCloseQuadSegments(TestImage image, Graphics2D graphics)
    {
        int width = image.getWidth() / 2;
        int height = image.getHeight();
        int size = Math.min(width, height);
        Path2D path = new Path2D.Double();
        path.moveTo(0.1 * size, 0.1 * size);
        path.quadTo(0.8 * size, 0.4 * size, 0.55 * size, 1.0 * size);
        path.quadTo(0.6 * size, 0.5 * size, 0.1 * size, 0.3 * size);
        path.quadTo(0.0 * size, 0.9 * size, 0.65 * size, 0.9 * size);
        path.closePath();
        drawPath(path, 0.1f * size, graphics, width);
        return TestResult.PASSED;
    }

    /**
     * Test drawing closed path constructed from cubic Bezier curve segments.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCloseCubicSegments(TestImage image, Graphics2D graphics)
    {
        int width = image.getWidth() / 2;
        int height = image.getHeight();
        int size = Math.min(width, height);
        Path2D path = new Path2D.Double();
        path.moveTo(0.1 * size, 0.1 * size);
        path.curveTo(0.8 * size, 0.4 * size, 0.8 * size, 0.9 * size, 0.55 * size, 1.0 * size);
        path.curveTo(0.6 * size, 0.5 * size, 0.65 * size, 0.9 * size, 0.1 * size, 0.3 * size);
        path.curveTo(0.0 * size, 0.9 * size, 0.65 * size, 0.9 * size, 0.55 * size, 1.0 * size);
        path.closePath();
        drawPath(path, 0.1f * size, graphics, width);
        return TestResult.PASSED;
    }

    /**
     * Test drawing closed path constructed from line segments, quadratic curves
     * and cubic curves.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCloseAllCombinations(TestImage image, Graphics2D graphics)
    {
        int width = image.getWidth() / 2;
        int height = image.getHeight();
        int size = Math.min(width, height);
        Path2D path = new Path2D.Double();
        path.moveTo(0.1 * size, 0.1 * size);
        path.lineTo(0.8 * size, 0.4 * size);
        path.quadTo(0.55 * size, 1.0 * size, 0.6 * size, 0.5 * size);
        path.curveTo(0.1 * size, 0.3 * size, 0.0 * size, 0.9 * size, 0.65 * size, 0.9 * size);
        path.closePath();
        drawPath(path, 0.1f * size, graphics, width);
        return TestResult.PASSED;
    }

    /**
     * Check if sharp turn is rendered correctly.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testSharpTurn1(TestImage image, Graphics2D graphics)
    {
        int width = image.getWidth() / 2;
        int height = image.getHeight();
        int size = Math.min(width, height);
        Path2D path = new Path2D.Double();
        path.moveTo(0, 0);
        path.curveTo(size, size, 0.4 * size, 0.6 * size, 0.1 * size, -0.36 * size);
        drawPath(path, 0.25f * size, graphics, width);
        return TestResult.PASSED;
    }

    /**
     * Check if sharp turn is rendered correctly.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testSharpTurn2(TestImage image, Graphics2D graphics)
    {
        int width = image.getWidth() / 2;
        int height = image.getHeight();
        int size = Math.min(width, height);
        Path2D path = new Path2D.Double();
        path.moveTo(0, 0);
        path.curveTo(size, size, 0.4 * size, 0.6 * size, 0.1 * size, -0.375 * size);
        drawPath(path, 0.25f * size, graphics, width);
        return TestResult.PASSED;
    }

    /**
     * Check if curve's cusp is rendered correctly.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testCusp(TestImage image, Graphics2D graphics)
    {
        int width = image.getWidth() / 2;
        int height = image.getHeight();
        int size = Math.min(width, height);
        Path2D path = new Path2D.Double();
        path.moveTo(0.1 * size, 0.1 * size);
        path.curveTo(0.9 * size, 0.9 * size, 0.1 * size, 0.9 * size, 0.9 * size, 0.1 * size);
        drawPath(path, 0.45f * size, graphics, width);
        return TestResult.PASSED;
    }

    /**
     * Entry point to the test suite.
     *
     * @param args not used in this case
     */
    public static void main(String[] args)
    {
        new SpecialCases().runTestSuite(args);
    }
}
